//
//  AppDelegate.m
//  iTNEFsEnough
//
//  Created by Josh Jacob on 3/22/12.
//  Copyright (c) 2012 Josh Jacob. All rights reserved.
//

#import "AppDelegate.h"

#import "LibytnefItem.h"
#import "LibytnefHelper.h"
#import "TNEFDataService.h"

@implementation AppDelegate

@synthesize window = _window;

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    // deal with file opens
    NSURL * fileToOpen = [launchOptions objectForKey:UIApplicationLaunchOptionsURLKey];
    if (fileToOpen != nil)
    {
        DLog(@"Called to open file %@", fileToOpen);
        bool delete = false;
        if ([launchOptions objectForKey:UIApplicationOpenURLOptionsOpenInPlaceKey] != nil) {
            delete = true;
        }
        
        [self openUrl:fileToOpen andDelete: delete];
    }
    
    [self updateFileList];
    
    // show "About" tab
    if (_files.count == 0) {
        if ([self.window.rootViewController isKindOfClass:[UITabBarController class]]) {
            UITabBarController *tabBar = (UITabBarController *) self.window.rootViewController;
            tabBar.selectedViewController = tabBar.viewControllers[1];
        }
    }
    
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
    /*
     Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
     Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
     */
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
    /*
     Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
     If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
     */
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
    /*
     Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
     */
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
    /*
     Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
     */
}

- (void)applicationWillTerminate:(UIApplication *)application
{
    /*
     Called when the application is about to terminate.
     Save data if appropriate.
     See also applicationDidEnterBackground:.
     */
}

- (UIInterfaceOrientationMask)application:(UIApplication *)application supportedInterfaceOrientationsForWindow:(UIWindow *)window
{
    return UIInterfaceOrientationMaskAll;
}

// DEPRECATED
/* - (BOOL)application:(UIApplication *)application
            openURL:(NSURL *)url
  sourceApplication:(NSString *)sourceApplication
         annotation:(id)annotation
{
    DLog(@"Open URL:\t%@\n"
         "With annotations:%@",
         url, annotation);
    
    [self openUrl:url];
    
    [self updateFileList];
    
    return YES;
} */

- (BOOL)application:(UIApplication *)app
            openURL:(NSURL *)url
            options:(NSDictionary *)options
{
    DLog(@"Open URL:\t%@\n"
          "With options:%@",
          url, options);
    
    bool delete = true;
    if ([options objectForKey:UIApplicationOpenURLOptionsOpenInPlaceKey] != nil) {
        delete = false;
    }
    
    [self openUrl:url andDelete: delete];
    
    [self updateFileList];
    
    return YES;
}

#pragma mark - Core Data

/**
 Returns the URL to the application's Documents directory.
 */
- (NSURL *)applicationDocumentsDirectory {
    return [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask] lastObject];
}

+(NSManagedObjectContext*) moc
{
	AppDelegate* delegate = (AppDelegate*)[[UIApplication sharedApplication] delegate];
	NSManagedObjectContext*  moc = [delegate managedObjectContext];
	
	return moc;
}

#pragma mark - Core Data stack

/**
 Returns the managed object context for the application.
 If the context doesn't already exist, it is created and bound to the persistent store coordinator for the application.
 */
- (NSManagedObjectContext *)managedObjectContext {
    
    if (managedObjectContext_ != nil) {
        return managedObjectContext_;
    }
    
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (coordinator != nil) {
        managedObjectContext_ = [[NSManagedObjectContext alloc] init];
        [managedObjectContext_ setPersistentStoreCoordinator:coordinator];
    }
    return managedObjectContext_;
}


/*
 Returns the managed object model for the application.
 If the model doesn't already exist, it is created from the application's model.
 */
- (NSManagedObjectModel *)managedObjectModel {
    
    if (managedObjectModel_ != nil) {
        return managedObjectModel_;
    }
    NSString *modelPath = [[NSBundle mainBundle] pathForResource:@"TNEFFiles" ofType:@"momd"];
    NSURL *modelURL = [NSURL fileURLWithPath:modelPath];
    managedObjectModel_ = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
    return managedObjectModel_;
}


/**
 Returns the persistent store coordinator for the application.
 If the coordinator doesn't already exist, it is created and the application's store added to it.
 */
- (NSPersistentStoreCoordinator *)persistentStoreCoordinator {
    
    if (persistentStoreCoordinator_ != nil) {
        return persistentStoreCoordinator_;
    }
    
    NSURL *storeURL = [[self applicationDocumentsDirectory] URLByAppendingPathComponent:@"TNEFFiles.sqlite"];
    
    NSDictionary *options = [NSDictionary dictionaryWithObjectsAndKeys:
                             [NSNumber numberWithBool:YES], NSMigratePersistentStoresAutomaticallyOption,
                             [NSNumber numberWithBool:YES], NSInferMappingModelAutomaticallyOption, nil];
    
    
    NSError *error = nil;
    persistentStoreCoordinator_ = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:[self managedObjectModel]];
    if (![persistentStoreCoordinator_ addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:storeURL options:options error:&error]) {
        /*
         Replace this implementation with code to handle the error appropriately.
         
         abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development. If it is not possible to recover from the error, display an alert panel that instructs the user to quit the application by pressing the Home button.
         
         Typical reasons for an error here include:
         * The persistent store is not accessible;
         * The schema for the persistent store is incompatible with current managed object model.
         Check the error message to determine what the actual problem was.
         
         
         If the persistent store is not accessible, there is typically something wrong with the file path. Often, a file URL is pointing into the application's resources directory instead of a writeable directory.
         
         If you encounter schema incompatibility errors during development, you can reduce their frequency by:
         * Simply deleting the existing store:
         [[NSFileManager defaultManager] removeItemAtURL:storeURL error:nil]
         
         * Performing automatic lightweight migration by passing the following dictionary as the options parameter:
         [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithBool:YES],NSMigratePersistentStoresAutomaticallyOption, [NSNumber numberWithBool:YES], NSInferMappingModelAutomaticallyOption, nil];
         
         Lightweight migration will only work for a limited set of schema changes; consult "Core Data Model Versioning and Data Migration Programming Guide" for details.
         
         */
        DLog(@"Unresolved error %@, %@", error, [error userInfo]);
        abort();
    }
    
    return persistentStoreCoordinator_;
}

#pragma mark -

- (void)openUrl:(NSURL *)url andDelete: (bool) delete
{
 
    CFStringGetFileSystemRepresentation( , <#char *buffer#>, <#CFIndex maxBufLen#>)
    
    LibytnefHelper * helper = [[LibytnefHelper alloc] initWithFile:url];
    if (helper.parseResult < 0) {
        
        // erase file
        if (delete) {
            NSError *error;
            [[NSFileManager defaultManager] removeItemAtURL:url error:&error];
            if (error != nil) {
                DLog(@"Error deleting bad file - %@", error);
            }
        }
        
        // alert user
        UIAlertView *parseErrorAlert = [[UIAlertView alloc] initWithTitle:@"Error Reading File!"
                                                                  message:@"This file wasn't able to be read."
                                                                 delegate:nil
                                                        cancelButtonTitle:@"OK"
                                                        otherButtonTitles:nil, nil];
        
        [parseErrorAlert show];
        
    } else {
    
        //
        // persist file in db
        //
        
        TNEFFileModel *tnefFile = [TNEFDataService createFileWithName:[url lastPathComponent]
                                                            fromEmail:helper.from
                                                              subject:helper.subject
                                                             dateSent:helper.date
                                                           bodyIsHTML:helper.bodyIsHTML];
        
        [TNEFDataService debugFile:tnefFile];
        
        //
        // create save directory 
        //
        
        NSString *saveDir = [NSHomeDirectory() stringByAppendingPathComponent:[NSString stringWithFormat:@"%@/%@/", TNEF_PATH_FILES, tnefFile.uuid]];
        NSFileManager *fileMgr = [NSFileManager defaultManager];
        if (![fileMgr fileExistsAtPath:saveDir isDirectory:nil])
        {
            NSError *error;
            BOOL created = [fileMgr createDirectoryAtPath:saveDir withIntermediateDirectories:YES attributes:nil error:&error];
            if (created == NO) {
                DLog(@"Directory not created: %@", error);
            }
        }
        
        //
        // write out body
        //
        
        if (helper.bodyIsHTML) {
            NSString *saveFile = [saveDir stringByAppendingPathComponent:@"body.html"];
            NSError *error;
            [helper.bodyHTML writeToFile:saveFile atomically:NO encoding:NSUTF8StringEncoding error:&error];
            if (error != nil) {
                DLog(@"Error writing out file: %@", error);
            }
        } else {
            NSString *saveFile = [saveDir stringByAppendingPathComponent:@"body.rtf"];
            NSError *error;
            // [helper.bodyRTF writeToFile:saveFile options:NSDataWritingFileProtectionNone error:&error];
            
            NSData *bodyRTFData = [helper.bodyRTF dataFromRange:(NSRange){0, [helper.bodyRTF length]} documentAttributes:@{NSDocumentTypeDocumentAttribute: NSRTFTextDocumentType} error:NULL];
            [bodyRTFData writeToFile:saveFile options:NSDataWritingFileProtectionNone error:&error];
            
            if (error != nil) {
                 DLog(@"Error writing out file: %@", error);
            }
        }
        
        //
        // write out files
        //
        
        int i = 0;
        for (id<LibytnefItem> file in helper.fileItems) {
            
            // create directory
            NSString *fileSaveDir = [saveDir stringByAppendingPathComponent:[NSString stringWithFormat:@"file%d", i]];
            NSError *error;
            BOOL created = [fileMgr createDirectoryAtPath:fileSaveDir withIntermediateDirectories:YES attributes:nil error:&error];
            if (created == NO) {
                DLog(@"File directory not created: %@", error);
            }
            
            // create nsdata
            NSData *data = [file fileData];
            NSString *filePath = [fileSaveDir stringByAppendingPathComponent:[file fileName]];
            [data writeToFile:filePath atomically:NO];
            
            i++;
        }
        
        //
        // copy original to save directory
        //
        
        if (delete == false) {
            NSString *movePath = [saveDir stringByAppendingPathComponent:@"original.dat"];
            NSError *error;
            [[NSFileManager defaultManager] moveItemAtURL:url
                                                    toURL:[NSURL fileURLWithPath:movePath]
                                                    error:&error];
            if (error != nil) {
                DLog(@"Error moving file - %@", error);
            }
        }
    }
}

-(void)updateFileList
{
    _files = [[TNEFDataService readAllFiles] mutableCopy];
    
    [[NSNotificationCenter defaultCenter] postNotificationName:NOTIFICATION_UPDATED_FILES object:self];
}

@end
